--
-- Class BuffManager
--
--   Manages a list of BuffManager

libAbilityCatalog = ...

local BuffManager = {}
BuffManager.__index = BuffManager

function BuffManager.Create()
	local obj = {}
	setmetatable(obj,BuffManager)
	obj:Init()
	return obj
end

function BuffManager:Init()
	self.units = {}

	self.watchedBuffIds = {}
	self.buffRegistration = {}
	libTDebug.UIDebug:AddTable("BuffManager.watchedBuffIds",self.watchedBuffIds)
	libTDebug.UIDebug:AddTable("BuffManager.buffRegistration",self.buffRegistration)
	libTDebug.UIDebug:AddTable("BuffManager.units",self.units)
end

function BuffManager:BuildWatchList()
	local newBuffs = {}
	for addonId, buffIds in pairs(self.buffRegistration)do
		for buffId, units in pairs(buffIds) do
			if (newBuffs[buffId] == nil)then
				newBuffs[buffId] = 0
			end
			newBuffs[buffId] = bit.bor(newBuffs[buffId], units)
		end
	end
	self.watchedBuffIds = newBuffs
	libTDebug.UIDebug:AddTable("BuffManager.watchedBuffIds",self.watchedBuffIds)
end

function BuffManager:UnregisterAll(addonId)
	self.buffRegistration[addonId] = nil
end

function BuffManager:Unregister(addonId,buffs)
	self.buffRegistration[addonId] = self.buffRegistration[addonId] or {}
	self.buffRegistration[addonId][buffId] = nil
end

function BuffManager:Register(addonId, buffId, units)
	self.buffRegistration[addonId] = self.buffRegistration[addonId] or {}
	if(not buffId) then return end
	if(self.buffRegistration[addonId][buffId] == nil)then
		self.buffRegistration[addonId][buffId] = 0
	end

	self.buffRegistration[addonId][buffId] = bit.bor(self.buffRegistration[addonId][buffId], units)
	self:BuildWatchList()
end

function BuffManager.UnitAdded(handle,unit)
	local self = libAbilityCatalog.BuffManager
	self.units[unit] = self.units[unit] or {}

	local chageTime = Inspect.Time.Frame()
	local buffIds = Inspect.Buff.List(unit)
	if(buffIds == nil) then return end
	local details = Inspect.Buff.Detail(unit, buffIds)
	for buffId, detail in pairs(details) do
		libAbilityCatalog.Event.Trigger.Buff(detail.type, unit, buffId)
		if(self.watchedBuffIds[detail.type] ~= nil)then
			if (bit.band(self.watchedBuffIds[detail.type],libAbilityCatalog.units[unit]) > 0) then
				self:AddChangeBuff(unit,detail,chageTime)
			end
		end
	end
end

function BuffManager.AddChangeBuffs(handle, unit, buffs)
	local self = libAbilityCatalog.BuffManager
	local buffIds = {}

	for buffId, buffTypeId in pairs(buffs) do
		if(buffTypeId == true)then
			-- udpate event
			table.insert(buffIds,buffId)
		else
			-- add event
			libAbilityCatalog.Event.Trigger.Buff(buffTypeId, unit, buffId)
			if(self.watchedBuffIds[buffTypeId] ~= nil and libAbilityCatalog.units[unit] ~= nil)then
			 	if (bit.band(self.watchedBuffIds[buffTypeId],libAbilityCatalog.units[unit]) > 0) then
					-- we need to watch this buff
					table.insert(buffIds,buffId)
				end
		  end
		end
	end

	local chageTime = Inspect.Time.Frame()
	local details = Inspect.Buff.Detail(unit,buffIds)

	if( #buffIds > 0 ) then
		-- process registered buffs
		for _, detail in pairs(details) do
			self:AddChangeBuff(unit,detail,chageTime)
		end
	end
end

function BuffManager:AddChangeBuff(unitId, detail, changeTime)
	if(self.units[unitId] == nil) then return end
	if(detail.type == nil) then return end
	detail.expires = detail.remaining ~= nil and detail.remaining + changeTime or 0
	detail.buffId = detail.id
	self.units[unitId][detail.type] = self.units[unitId][detail.type] or {}
	self.units[unitId].buffMap = self.units[unitId].buffMap or {}
	self.units[unitId].buffMap[detail.id] = detail.type

	if(detail.caster == libAbilityCatalog.handledSpecifiers["player"])then
		-- self cast
		self.units[unitId][detail.type].player = detail
	end
	-- anyone cast
	if(self.units[unitId][detail.type].others == nil or
	 	 (self.units[unitId][detail.type].others ~= nil and
		 detail.expires > self.units[unitId][detail.type].others.expires))then
		self.units[unitId][detail.type].others = detail
	end
end

function BuffManager.BuffRemove(handle, unit, buffs)
	local self = libAbilityCatalog.BuffManager
	if not self.units[unit] then return end
  if not self.units[unit].buffMap then return end
	for id, _ in pairs(buffs) do
		local buffTypeId = self.units[unit].buffMap[id]
		if(self.units[unit][buffTypeId] ~= nil)then
			if(self.units[unit][buffTypeId].player and self.units[unit][buffTypeId].player.buffId == id)then
				self.units[unit][buffTypeId].player = nil
			end
			if(self.units[unit][buffTypeId].others and self.units[unit][buffTypeId].others.buffId == id)then
				self.units[unit][buffTypeId].others = nil
			end
			if(self.units[unit][buffTypeId].others == nil and self.units[unit][buffTypeId].player == nil)then
				self.units[unit][buffTypeId] = nil
				self.units[unit].buffMap[id] = nil
			end
		end
	end
end

function BuffManager.UnitRemoved(handle,unit)
	local self = libAbilityCatalog.BuffManager
	if(libAbilityCatalog.handledSpecifiers["player"] ~= unit)then
		self.units[unit] = nil
	end
end

Command.Event.Attach(Event.Buff.Add, BuffManager.AddChangeBuffs, "Event.Buff.Add")
Command.Event.Attach(Event.Buff.Remove, BuffManager.BuffRemove, "Event.Buff.Remove")
Command.Event.Attach(Event.Buff.Change, BuffManager.AddChangeBuffs, "Event.Buff.Change")
Command.Event.Attach(libAbilityCatalog.Event.UnitRemoved, BuffManager.UnitRemoved, "BuffManager.UnitRemoved")
Command.Event.Attach(libAbilityCatalog.Event.UnitAdded, BuffManager.UnitAdded, "BuffManager.UnitAdded")

libAbilityCatalog.BuffManager = BuffManager.Create()

