--
-- Class Notifications
--
--   Manages a list of Notifications

local addonData, triggers = ...

local Notifications = {}
Notifications.__index = Notifications

function Notifications.Create()
	local obj = {}
	setmetatable(obj,Notifications)
	obj:Init()
	return obj
end

function Notifications:Init()
	self.items = {}
	libTDebug.UIDebug:AddTable("Notifications.items",self.items)
	self.isEditMode = false
	self.filter = {}
	self.filter.Spec = function()
		return nil
	end
end

function Notifications:AddNotification(n)
	self.items[n.id] = n
	n:UISetup(self.parent)
end

function Notifications:AddNotificationFromVariable(value)
	local n = triggers.Notification.Deserialize(value)
	self.items[n.id]=n
	n:UISetup(self.parent)
end

function Notifications.UISetup(handle,parent)
	local self = triggers.Notifications
	self.parent = parent
	for _, value in pairs(TRIGGER_NOTIFICATIONS) do
		self:AddNotificationFromVariable(value)
	end
	Triggers.Event.Trigger.NotificationsLoaded()
end

function Notifications.OnVariablesLoaded(event, addonId)
	local self = triggers.Notifications
	if addonId ~= triggers.addonId then return end
	TRIGGER_NOTIFICATIONS = TRIGGER_NOTIFICATIONS or {}
end

function Notifications.OnVariablesSaving(event, addonId)
	if addonId ~= triggers.addonId then return end
	local self = triggers.Notifications
	TRIGGER_NOTIFICATIONS = TRIGGER_NOTIFICATIONS or {}
	for id, item in pairs(self.items) do
		if(item.remove)then
			TRIGGER_NOTIFICATIONS[id] = nil
		else
			TRIGGER_NOTIFICATIONS[id] = item:Serialize()
		end
	end
end

function Notifications.OnSlash(handle, parameter)
	local self = triggers.Notifications
	if (self.SlashCommands[parameter] ~= nil) then
		self.SlashCommands[parameter][1]()
	else
		for key, value in pairs(self.SlashCommands) do
			print(string.format("/Notifications %s - %s", key, value[2]))
		end
	end
end

function Notifications:GetUniqueRoles()
	local notificationRoles = {}
	for _, n in pairs(self.items) do
		local individualRoles = libTUtility.StringSplit(n.roles, ",")
		for _,v in ipairs(individualRoles) do
			notificationRoles[libTUtility.Trim(string.gsub(v,",",""))] = true
		end
	end
	local retVal = {}
	for v,_ in pairs(notificationRoles)do
		table.insert(retVal,v)
	end

	local sort_func = function( a,b ) return a < b end
	table.sort( retVal, sort_func )

	table.insert(retVal, 1, "All Roles")

	return retVal
end

function Notifications:GetByNoRolePopulated()
	local matchedNotifications = {}

	for _, n in pairs(self.items) do
		if (string.len(n.roles) <= 0 and n.deletable ~= false)  then
			table.insert(matchedNotifications,n)
		end
	end
	return matchedNotifications
end

function Notifications:GetByRoleMatch(role)
	local matchedNotifications = {}
	local specPattern = ".*" .. role .. ".*"
	for _, n in pairs(self.items) do
		if (string.find(n.roles, specPattern) and n.deletable ~= false) then
			table.insert(matchedNotifications,n)
		end
	end
	return matchedNotifications
end

function Notifications:Delete(id)
	self:DeleteNotification(self.items[id])
end

function Notifications:DeleteNotification(notification)
	notification:Delete()
end

function Notifications:CreateNewNotification(imageSelect,top, left, size, openConfig)
	if(imageSelect)then
		triggers.ImageScroller:Show(true)
		triggers.Notifications:EditMode(false)
		function triggers.ImageScroller:ImageSelected(resource, image)
			local n = triggers.Notification.Create(resource, image)
			n:UISetup(triggers.Notifications.parent)
			n.abilityIcon = false
			if(top) then n:SetTop(top) end
			if(left) then n:SetLeft(left) end
			if(size) then n:SetSize(size) end
			triggers.Notifications.items[n.id] = n
			triggers.Notifications:EditMode(true)
			if(openConfig)then
				triggers.Configuration.Notifications.Config:Show(n)
			end
		end
	else
		local n = triggers.Notification.Create()
		n:UISetup(self.parent)
		if(top) then n:SetTop(top) end
		if(left) then n:SetLeft(left) end
		if(size) then n:SetSize(size) end
		triggers.Notifications.items[n.id] = n
		triggers.Notifications:EditMode(true)
		if(openConfig)then
			triggers.Configuration.Notifications.Config:Show(n)
		end
	end
end

function Notifications:GetTrigger(triggers,id)
	for _, t in ipairs(triggers)do
		if(t.id == id)then
			return t
		end
	end
	return nil
end

function Notifications:GetNotification(id)
	return self.items[id]
end

function Notifications:UpdateNotification(section, version)
	local n = self.items[section.id]
	if(n)then
		-- triggers from default
		if(section.triggers)then
			for _,item in ipairs(section.triggers)do
				local t = n:GetTriggerById(item.id)
				local exist = t and true or false
				triggers.Trigger.Deserialize(item,t)
				if(not exists)then
					n:AppendTrigger(t)
				end
			end
		end
	end
end

function Notifications:HideOthersWhileConfigerz(notification)
	for _, n in pairs(self.items) do
		if(notification ~= n)then
			n:EditMode(false)
		end
	end
end

function Notifications:ShowOthersAfterConfigerz()
	local filterRole = self.filter:Spec()
	for _, n in pairs(self.items) do
		if(n:SpecMatch(filterRole or self:Spec()) and not n.remove)then
			n:EditMode(true)
		end
	end
end

function Notifications:FindLikeEdge(notification)
	local topRightCorner = { x = notification.position.left + notification.size, y = notification.position.top }
	local topLeftCorner = { x = notification.position.left, y = notification.position.top }
	local bottomRightCorner = { x = notification.position.left + notification.size, y = notification.position.top + notification.size }
	local bottomLeftCorner = { x = notification.position.left, y = notification.position.top + notification.size }
	local filterRole = self.filter:Spec()
	for _, n in pairs(self.items) do
		if(n ~= notification and not n.disabled and n:SpecMatch(filterRole or self:Spec()))then
			local fn = n:CheckRegion(notification.size,topLeftCorner,topRightCorner,bottomLeftCorner,bottomRightCorner)
			if(fn) then
				return fn
			end
		end
	end
	return nil
end

function Notifications:EditMode(enabled)
	self.isEditMode = enabled
	local filterRole = self.filter:Spec()
	triggers.ExpressionsManager:EnableExpressions(not self.isEditMode)
	for _, notification in pairs(self.items) do
		if(notification:SpecMatch(filterRole or self:Spec())  and not notification.remove)then
			notification:EditMode(self.isEditMode)
		end
	end
end

function Notifications:ToggleEdit()
	self:EditMode(not self.isEditMode)
end

function Notifications:HideNotificitions()
	for _, n in pairs(self.items) do
		n:SetViewState()
	end
end

function Notifications:Spec()
		local roleList = Inspect.Role.List()
		local slot = Inspect.TEMPORARY.Role()
		if(slot)then
			self.role = roleList[string.format("h%016X", slot - 1)]
		end
	  return self.role
end


function Notifications:SpecDisplayFilter(filter)
	self.filter = filter
end

triggers.Notifications = Notifications.Create()

Command.Event.Attach(Event.Addon.SavedVariables.Load.End, Notifications.OnVariablesLoaded, "Event.Addon.SavedVariables.Load.End")
Command.Event.Attach(Event.Addon.SavedVariables.Save.Begin, Notifications.OnVariablesSaving, "Event.Addon.SavedVariables.Save.Begin")

Command.Event.Attach(Command.Slash.Register("Notifications"), Notifications.OnSlash, "SlashRegisterCommand.Notifications")
Command.Event.Attach(Triggers.Event.ContextCreated , Notifications.UISetup, "Notifications.UISetup")
